#ifndef CHROMEOS_CAMERA_LIB_MEDIAPIPE_SHIM_FRAMEWORK_CALCULATOR_GRAPH_H_
#define CHROMEOS_CAMERA_LIB_MEDIAPIPE_SHIM_FRAMEWORK_CALCULATOR_GRAPH_H_

#include <functional>
#include <map>
#include <memory>
#include <optional>
#include <string>

#include "chromeos/camera/lib/mediapipe_shim/framework/calculator_framework.h"

namespace mediapipe_shim {

// Analogous to mediapipe::CalculatorGraph.
class CalculatorGraph {
 public:
  static std::unique_ptr<CalculatorGraph> Create();

  virtual ~CalculatorGraph() = default;
  virtual bool Initialize(
      const std::string& graph_config,
      const std::map<std::string, Packet>& side_packets) = 0;
  virtual bool ObserveOutputStream(
      const std::string& stream_name,
      std::function<bool(Packet)> packet_callback) = 0;
  virtual std::optional<Packet> GetOutputSidePacket(
      const std::string& packet_name) = 0;
  virtual bool StartRun(
      const std::map<std::string, Packet>& extra_side_packets) = 0;
  virtual bool WaitUntilDone() = 0;
  virtual bool WaitUntilIdle() = 0;
  virtual bool HasError() const = 0;
  virtual bool AddPacketToInputStream(const std::string& stream_name,
                                      Packet packet) = 0;
  virtual bool CloseAllInputStreams() = 0;
  virtual void Cancel() = 0;
};

}  // namespace mediapipe_shim

#endif  // CHROMEOS_CAMERA_LIB_MEDIAPIPE_SHIM_FRAMEWORK_CALCULATOR_GRAPH_H_
