#ifndef CHROMEOS_CAMERA_LIB_MEDIAPIPE_SHIM_FRAMEWORK_FORMATS_IMAGE_FRAME_H_
#define CHROMEOS_CAMERA_LIB_MEDIAPIPE_SHIM_FRAMEWORK_FORMATS_IMAGE_FRAME_H_

#include <cstdint>
#include <functional>
#include <memory>

namespace mediapipe {
class ImageFrame;
};

namespace mediapipe_shim {

// Analogous to mediapipe::ImageFormat::Format.
enum ImageFormat {
  kSRGB,
  kSRGBA,
  kGRAY8,
};

// Analogous to mediapipe::ImageFrame.
class ImageFrame {
 public:
  ImageFrame(ImageFormat format, int width, int height, int width_step,
             uint8_t* pixel_data, std::function<void(uint8_t*)> deleter);
  ImageFrame(ImageFrame&) = delete;
  ImageFrame& operator=(ImageFrame&) = delete;
  ImageFrame(ImageFrame&&);
  ImageFrame& operator=(ImageFrame&&);
  ~ImageFrame();

  bool IsEmpty() const;
  void SetToZero();
  ImageFormat Format() const;
  int Width() const;
  int Height() const;
  int NumberOfChannels() const;
  int ByteDepth() const;
  int WidthStep() const;
  uint8_t* MutablePixelData();
  const uint8_t* PixelData() const;

 private:
  friend class ToInternalImageFrameCalculator;
  friend class ToShimImageFrameCalculator;

  explicit ImageFrame(std::unique_ptr<mediapipe::ImageFrame> image_frame);
  [[nodiscard]] mediapipe::ImageFrame* release() {
    return image_frame_.release();
  }

  std::unique_ptr<mediapipe::ImageFrame> image_frame_;
};

}  // namespace mediapipe_shim

#endif  // CHROMEOS_CAMERA_LIB_MEDIAPIPE_SHIM_FRAMEWORK_FORMATS_IMAGE_FRAME_H_
