#ifndef CHROMEOS_CAMERA_LIB_MEDIAPIPE_SHIM_GPU_GPU_BUFFER_H_
#define CHROMEOS_CAMERA_LIB_MEDIAPIPE_SHIM_GPU_GPU_BUFFER_H_

#include <GLES2/gl2.h>

#include <memory>
#include <string>

namespace mediapipe {
class GlTextureView;
class GpuBuffer;
}  // namespace mediapipe

namespace mediapipe_shim {

enum class GpuBufferFormat {
  kBGRA32,
  kRGBA32,
  kOneComponent8,
  kTwoComponent8,
  kNV12,
};

class GlTextureView {
 public:
  GlTextureView(GlTextureView&) = delete;
  GlTextureView& operator=(GlTextureView&) = delete;
  ~GlTextureView();

  int width() const;
  int height() const;
  GLenum target() const;
  GLuint name() const;
  int plane() const;

 private:
  friend class GpuBuffer;

  explicit GlTextureView(std::unique_ptr<mediapipe::GlTextureView> view);

  std::unique_ptr<mediapipe::GlTextureView> view_;
};

class GpuBuffer {
 public:
  // Analogous to mediapipe::GpuBuffer(mediapipe::GlTextureBuffer::Wrap(...)).
  static GpuBuffer WrapGlTexture(GLenum target, GLuint name, int width,
                                 int height, GpuBufferFormat format);

  GpuBuffer(int width, int height, GpuBufferFormat format);
  GpuBuffer(GpuBuffer&) = delete;
  GpuBuffer& operator=(GpuBuffer&) = delete;
  GpuBuffer(GpuBuffer&&);
  GpuBuffer& operator=(GpuBuffer&&);
  ~GpuBuffer();

  int width() const;
  int height() const;
  GpuBufferFormat format() const;
  explicit operator bool() const;
  std::string DebugString() const;

  // Analogous to GetReadView<GlTextureView>() / GetWriteView<GlTextureView>().
  GlTextureView GetGlTextureView(int plane) const;

 private:
  friend class ToInternalGpuBufferCalculator;
  friend class ToShimGpuBufferCalculator;

  explicit GpuBuffer(std::unique_ptr<mediapipe::GpuBuffer> gpu_buffer);
  [[nodiscard]] mediapipe::GpuBuffer* release() {
    return gpu_buffer_.release();
  }

  std::unique_ptr<mediapipe::GpuBuffer> gpu_buffer_;
};

}  // namespace mediapipe_shim

#endif  // CHROMEOS_CAMERA_LIB_MEDIAPIPE_SHIM_GPU_GPU_BUFFER_H_
